/**
 * Author   : Davide Ficano
 * Date     : 13-Mar-2005 Fixed context/view menu retrieving for Mozilla/Thunderbird
 *                        Added ability to save DOM document
 * Date     : 20-Mar-2005 Changed call to runProgram (Contributor Johannes Dollinger)
 *                        Added listener to saveURI to save correctly big pages
 */

var gViewSourceWithMain = {

    onLoad : function() {
        var thiz = gViewSourceWithMain;
        thiz.prefs = new ViewSourceWithPrefs();

        // Under firefox frame menupopup hasn't an id to overlay 
        // so I use this workaround. Mozilla has a valid frame menupopup id
        var frameMenu = document.getElementById("frame");
        if (frameMenu) {
            frameMenu.addEventListener("popupshowing", thiz.initCtxMenu, false);
        }
        
        thiz.gEditorData = null;
        thiz.gEditorDefaultIndex = -1;
        
        var obs = ViewSourceWithCommon.getObserverService();
        obs.addObserver(thiz, 
                        "viewsource-with-configuration-changed", false);
        obs.notifyObservers(null, "viewsource-with-configuration-changed", "");
    },

    initCtxMenu : function(event) {
        var frameMenu = document.getElementById("frame");
        var isInFrame = frameMenu && frameMenu.hidden == false;
        //var isInFrame = gContextMenu.target.ownerDocument != window._content.document;
        var fnName = "viewPageFromCtxMenu";

        if (isInFrame && frameMenu.firstChild) {
            var popup = document.getElementById("vswFramePopupMenu");
            fnName = "viewPageFromCtxMenuFrameset";
            
            // Create popup if it not already exists
            if (popup == null) {
                var menu = document.createElement("menu");
                var label = document.getElementById("viewsourcewithMenu")
                                .getAttribute("framelabel");
                menu.setAttribute("label", label);
                popup = document.createElement("menupopup");
                popup.setAttribute("id", "vswFramePopupMenu");
                menu.appendChild(popup);
                frameMenu.firstChild.appendChild(menu);
            }
            
            gViewSourceWithMain.insertMenuItems(popup, "viewPageFromCtxMenu");
        }
        gViewSourceWithMain.insertMenuItems(
                document.getElementById("viewsourcewithMenuPopup"), 
                fnName);
        return true;
    },
    
    initViewMenu : function(event) {
        gViewSourceWithMain.insertMenuItems(
                document.getElementById("viewsourcewith-viewMenuPopup"), 
                "viewPageFromViewMenu");
        return true;
    },

    initToolbarMenu : function(event) {
        gViewSourceWithMain.insertMenuItems(
                document.getElementById("viewsourcewith-toolbarMenuPopup"), 
                "viewPageFromViewMenu");
        return true;
    },

    viewPageFromCtxMenuFrameset : function(editorDataIdx, event) {
        // use the whole document to access to frameset
        gViewSourceWithMain.viewPage(window._content.document, editorDataIdx, event);
    },

    viewPageFromCtxMenu : function(editorDataIdx, event) {
        // access to the frame underneath cursor
        gViewSourceWithMain.viewPage(gContextMenu.target.ownerDocument, editorDataIdx, event);
    },
    
    viewPageFromViewMenu : function(editorDataIdx, event) {
        var showFrameWarning = gViewSourceWithMain.prefs.showFrameWarning;
        
        if (showFrameWarning) {
            gViewSourceWithMain.openDlgWarning();
        }
        
        var focusedWindow = document.commandDispatcher.focusedWindow;
        // Don't get url from browser widgets (e.g. google bar, address bar)
        if (focusedWindow == window) {
            focusedWindow = _content;
        }

        gViewSourceWithMain.viewPage(focusedWindow.document, editorDataIdx, event);
    },

    viewPage : function(documentToSave, editorDataIdx, event) {
        try {
            var urlToSave       = documentToSave.location.href;//documentToSave.URL;
            var filePath;
            var thiz            = gViewSourceWithMain;
            var editorPath      = thiz.gEditorData[editorDataIdx].path;

            if (editorPath == "") {
                return;
            }
            var fileEditor = ViewSourceWithCommon.makeLocalFile(editorPath);
            if (thiz.isLocalPage(urlToSave)) {
                // view file without save it
                filePath = ViewSourceWithCommon.makeLocalFileByUrl(urlToSave);
                ViewSourceWithCommon.launchApp(fileEditor, filePath);
            } else {
                filePath = ViewSourceWithCommon.makeLocalFile(
                                    thiz.prefs.destFolder, 
                                    [thiz.getFileName(documentToSave)]);
                var saveDOM = (thiz.prefs.saveMethod == "dom") != event.shiftKey;
                var saver = new ViewSourceWithSaver(
                                        documentToSave, fileEditor, filePath);
                if (saveDOM) {
                    saver.saveDocument();
                } else {
                    saver.saveURI();
                }
            }
        } catch (err) {
            alert("viewPage: " + err);
        }
    },
    
    openDlgSettings : function() {
        window.openDialog("chrome://viewsourcewith/content/settings/settings.xul", 
                          "_blank", 
                          "chrome,resizable=yes,dependent=yes");
    },
    
    openDlgWarning : function() {
        window.openDialog("chrome://viewsourcewith/content/warning-frame.xul", 
                          "_blank", 
                          "chrome,modal,resizable=yes,dependent=yes");
    },
    
    isLocalPage : function(url) {
        return url.substring(0, 7) == "file://";
    },
    
    getFileName : function(doc) {
        const contractIDStdURL = "@mozilla.org/network/standard-url;1";
        const nsIURL = Components.interfaces.nsIURL;
        var uri = Components.classes[contractIDStdURL].createInstance(nsIURL);
    
        uri.filePath = doc.location.href;

        var fileName;
        // This url contains only directory name (e.g. http://www.xulplanet.com/)
        var re = /\/([^\/]+)\/$/;
        var path = uri.path.match(re);
        if (path && path.length > 1) {
            fileName = path[1];
        } else {
            fileName = uri.fileName;
        }
        // Handle cases like 
        // http://host.com/path/?noFileName=before_question_mark_no_name
        if (fileName == "") {
            path = uri.directory.match(re);
            if (path && path.length > 1) {
                fileName = path[1];
            } else {
                fileName = "index";
            }
        }
        // Check if fileName ends with .htm or .html
        re = /\.(htm|html|xml)$/i;
        if (!fileName.match(re)) {
            fileName += "." + ViewSourceWithCommon.getMIMEService()
                                    .getPrimaryExtension(doc.contentType, null)
        }
        return gViewSourceWithMain.getPortableFileName(fileName);
    },
    
    /**
     * Replace following chars with underscore (_)
     * \ / : * ? " < > |
     */
    getPortableFileName : function(fileName) {
        return fileName.replace(/[\\\/\:\*\?\"\<\>\| ]+/g, "_"); //"
    },
    
    insertMenuItems : function(menu, fnViewPage) {
        if (!menu) {
            return;
        }
        var thiz = gViewSourceWithMain;

        thiz.removeMenuItems(menu);

        thiz.gEditorData = thiz.prefs.editorData;
        thiz.gEditorDefaultIndex = thiz.prefs.editorDefaultIndex;
        
        thiz.insertDefaultItemMenu(menu, fnViewPage);

        var hasVisibleItems = false;
        for (var i = 0; i < thiz.gEditorData.length; i++) {
            if (i != thiz.gEditorDefaultIndex && thiz.gEditorData[i].isVisible) {
                hasVisibleItems = true;
                break;
            }
        }

        // add separator only if there is at least one editor visible
        if (hasVisibleItems) {
            menu.appendChild(document.createElement("menuseparator"));
        }

        for (i = 0; i < thiz.gEditorData.length; i++) {
            if (i != thiz.gEditorDefaultIndex && thiz.gEditorData[i].isVisible) {
                thiz.insertEditorMenuItem(menu, i, fnViewPage);
            }
        }
        var hasDefault = thiz.gEditorDefaultIndex >= 0;
        thiz.insertSettingsMenuItem(menu, !hasDefault);
    },

    removeMenuItems : function(menu) {
        var children = menu.childNodes;
        
        for (var i = children.length - 1; i >= 0; i--) {
            menu.removeChild(children[i]);
        }
    },

    insertDefaultItemMenu : function(menu, fnViewPage) {
        var thiz = gViewSourceWithMain;
        var hasDefault = thiz.gEditorDefaultIndex >= 0;

        if (hasDefault) {
            var item = document.createElement("menuitem");
            
            item.setAttribute("label", thiz.gEditorData[thiz.gEditorDefaultIndex].description);
            item.setAttribute("oncommand", "gViewSourceWithMain." 
                                    + fnViewPage + "(" + thiz.gEditorDefaultIndex + ", event);");
            item.setAttribute("id", "viewsourcewith-viewMenuItemDefault");
            // setting key after appendChild doesn't work
            item.setAttribute("key", "key_viewsourcewith");
            menu.appendChild(item);
        }
    },

    insertSettingsMenuItem : function(menu, hasShortCutKey) {
        var item = document.createElement("menuitem");
        
        item.setAttribute("label", 
                ViewSourceWithCommon.getLocalizedMessage("menu.settings.label"));
        item.setAttribute("oncommand", "gViewSourceWithMain.openDlgSettings();");
        if (hasShortCutKey) {
            item.setAttribute("id", "viewsourcewith-viewMenuItemDefault");
            item.setAttribute("key", "key_viewsourcewith");
        }
        menu.appendChild(item);
        
        return item;
    },
    
    insertEditorMenuItem : function(menu, editorDataIdx, fnViewPage) {
        var item = document.createElement("menuitem");
        var label = gViewSourceWithMain.gEditorData[editorDataIdx].description;
        
        item.setAttribute("id", "viewsourcewithEditor" + label);
        item.setAttribute("label", label);
        item.setAttribute("oncommand", "gViewSourceWithMain." 
                                + fnViewPage + "(" + editorDataIdx + ", event);");
        menu.appendChild(item);

        return item;
    },
    
    runDefaultEditor : function(event) {
        var thiz = gViewSourceWithMain;

        thiz.gEditorData = thiz.prefs.editorData;
        thiz.gEditorDefaultIndex = thiz.prefs.editorDefaultIndex;

        if (thiz.gEditorDefaultIndex < 0) {
            thiz.openDlgSettings();
        } else {
            thiz.viewPageFromViewMenu(thiz.gEditorDefaultIndex, event);
        }
    },

    observe : function(subject, topic, state) {
        if (topic == "viewsource-with-configuration-changed") {
            // ensure shortkeys are assigned
            gViewSourceWithMain.initViewMenu(null);
        }
    }
};

function ViewSourceWithSaver(documentToSave, editorFile, appFile) {
    this.documentToSave = documentToSave;
    this.editorFile = editorFile;
    this.appFile = appFile;
}

ViewSourceWithSaver.prototype = {
    saveDocument : function() {
        const nsIWBP = Components.interfaces.nsIWebBrowserPersist;
        var persist = ViewSourceWithCommon.makeWebBrowserPersist();

        persist.progressListener = this;
        persist.persistFlags = nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES
                               | nsIWBP.PERSIST_FLAGS_FROM_CACHE
                               | nsIWBP.PERSIST_FLAGS_NO_CONVERSION
                               | nsIWBP.PERSIST_FLAGS_NO_BASE_TAG_MODIFICATIONS
                               | nsIWBP.PERSIST_FLAGS_DONT_FIXUP_LINKS;
        var encodingFlags = nsIWBP.ENCODE_FLAGS_RAW;
        persist.saveDocument(this.documentToSave, this.appFile, 
                             null, null, encodingFlags, 0);
    },

    saveURI : function() {
        const nsIWBP = Components.interfaces.nsIWebBrowserPersist;
        var persist = ViewSourceWithCommon.makeWebBrowserPersist();

        persist.progressListener = this;
        persist.persistFlags = nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES
                               | nsIWBP.PERSIST_FLAGS_FROM_CACHE;

        var referrer = ViewSourceWithBrowserHelper.getReferrer(document);
        
        var uri = ViewSourceWithCommon.makeURL(this.documentToSave.location.href);
        persist.saveURI(uri, null, referrer, 
                        ViewSourceWithBrowserHelper.getPostData(), 
                        null, this.appFile);
    },

    onStateChange : function(webProgress, request, stateFlags, status) {
        const wpl = Components.interfaces.nsIWebProgressListener;
        var isLoadFinished = (stateFlags & wpl.STATE_STOP)
                             ;//&& (stateFlags & wpl.STATE_IS_NETWORK);
        
        if (isLoadFinished) {
            ViewSourceWithCommon.launchApp(this.editorFile, this.appFile);
        }
    },

    QueryInterface : function(iid) {
        if (iid.equals(Components.interfaces.nsIWebProgressListener) ||
            iid.equals(Components.interfaces.nsISupports)) {
            return this;
        }

        throw Components.results.NS_NOINTERFACE;
    },

    onStatusChange : function(webProgress, request, status, message) {},
    onLocationChange : function(webProgress, request, location) {},
    onProgressChange : function(webProgress, request, 
                                curSelfProgress, maxSelfProgress, 
                                curTotalProgress, maxTotalProgress) {},
    onSecurityChange : function(webProgress, request, state) {},
}

window.addEventListener("load", gViewSourceWithMain.onLoad, false);
